//-------------------------------------------------------------------------------------

//              GHI Electronics, LLC

//               Copyright (c) 2010

//               All rights reserved

//-------------------------------------------------------------------------------------

/*

 * You can use this file if you agree to the following:

 *

 * 1. This header can't be changed under any condition.

 *    

 * 2. This is a free software and therefore is provided with NO warranty.

 * 

 * 3. Feel free to modify the code but we ask you to provide us with

 *	  any bugs reports so we can keep the code up to date.

 *

 * 4. This code may ONLY be used with GHI Electronics, LLC products.

 *

 * THIS SOFTWARE IS PROVIDED BY GHI ELECTRONICS, LLC ``AS IS'' AND 

 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 

 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 

 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL 

 * GHI ELECTRONICS, LLC BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,

 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 

 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,

 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 

 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR ORT 

 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 

 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  

 *

 *	Specs are subject to change without any notice

 */



using System;

using System.Threading;

using Microsoft.SPOT.Hardware;



using GHIElectronics.NETMF.Hardware;





namespace GHIElectronics.NETMF.FEZ

{

    public static partial class FEZ_Components

    {

        public class DCMotorDriver : IDisposable

        {



            PWM _pwm1;

            OutputPort _dir1a;

            OutputPort _dir1b;



            PWM _pwm2;

            OutputPort _dir2a;

            OutputPort _dir2b;



            sbyte _last_speed1, _last_speed2;



            public void Dispose()

            {

                _pwm1.Dispose();

                _dir1a.Dispose();

                _dir1b.Dispose();



                _pwm2.Dispose();

                _dir2a.Dispose();

                _dir2b.Dispose();

            }



            public DCMotorDriver(FEZ_Pin.PWM PWM1, FEZ_Pin.Digital DIR1A, FEZ_Pin.Digital DIR1B,

                FEZ_Pin.PWM PWM2, FEZ_Pin.Digital DIR2A, FEZ_Pin.Digital DIR2B)

            {

                _pwm1 = new PWM((PWM.Pin)PWM1);

                _dir1a = new OutputPort((Cpu.Pin)DIR1A, false);

                _dir1b = new OutputPort((Cpu.Pin)DIR1B, false);



                _pwm2 = new PWM((PWM.Pin)PWM2);

                _dir2a = new OutputPort((Cpu.Pin)DIR2A, false);

                _dir2b = new OutputPort((Cpu.Pin)DIR2B, false);

            }

            public void Move(sbyte speed1, sbyte speed2)

            {

                if (speed1 > 100 || speed1 < -100 || speed2 > 100 || speed2 < -100)

                    new ArgumentException();



                _last_speed1 = speed1;

                _last_speed2 = speed2;





                if (speed1 < 0)

                {

                    _dir1a.Write(true);

                    _dir1b.Write(false);



                }

                else

                {

                    _dir1a.Write(false);

                    _dir1b.Write(true);

                }

                _pwm1.Set(1000, (byte)Math.Abs(speed1));



                ////////////////////////////



                if (speed2 < 0)

                {

                    _dir2a.Write(true);

                    _dir2b.Write(false);



                }

                else

                {

                    _dir2a.Write(false);

                    _dir2b.Write(true);

                }

                _pwm2.Set(1000, (byte)Math.Abs(speed2));









            }

            public void MoveRamp(sbyte speed1, sbyte speed2, byte ramping_step_delay_mSec)

            {

                sbyte temp_speed1 = _last_speed1;

                sbyte temp_speed2 = _last_speed2;



                while ((speed1 != temp_speed1) ||

                        (speed2 != temp_speed2))

                {

                    if (temp_speed1 > speed1)

                        temp_speed1--;

                    if (temp_speed1 < speed1)

                        temp_speed1++;



                    if (temp_speed2 > speed2)

                        temp_speed2--;

                    if (temp_speed2 < speed2)

                        temp_speed2++;



                    Move(temp_speed1, temp_speed2);

                    Thread.Sleep(ramping_step_delay_mSec);

                }

            }



        }

    }

}

