using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.Hardware;
using GHIElectronics.NETMF.FEZ;

namespace GHIElectronics.NETMF.FEZ
{
    public static partial class FEZ_Components
    {
        public class PCF8574 : IDisposable
        {
            I2CDevice I2C;
            I2CDevice.I2CTransaction[] WriteXaction;
            I2CDevice.I2CTransaction[] ReadXaction;
            byte[] DataRead;
            byte[] DataWrite;

            public void Dispose()
            {
                I2C.Dispose();
                WriteXaction = null;
                ReadXaction = null;
                DataRead = null;
                DataWrite = null;
            }

            public PCF8574(byte address)
            {
                if (address > 7)
                    throw new ArgumentException("address");

                I2C = new I2CDevice(new I2CDevice.Configuration((ushort)(0x20 | address), 100));

                WriteXaction = new I2CDevice.I2CTransaction[1];
                DataWrite = new byte[1];
                WriteXaction[0] = I2CDevice.CreateWriteTransaction(DataWrite);

                ReadXaction = new I2CDevice.I2CTransaction[1];
                DataRead = new byte[64];
                ReadXaction[0] = I2CDevice.CreateReadTransaction(DataRead);

            }
            public void WritePort(byte value)
            {
                WriteXaction[0].Buffer[0] = value;
                int i = I2C.Execute(WriteXaction, 100);
                if (i == 0)
                {
                    new Exception("I2C Failed");
                }
            }
            public byte[] ReadPort()
            {
                int i = I2C.Execute(ReadXaction, 100);
                if (i == 0)
                {
                    new Exception("I2C Failed");
                }
                return ReadXaction[0].Buffer;
            }


        }
    }
}
